/* valasignaltype.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <vala/valasignaltype.h>




struct _ValaSignalTypePrivate {
	ValaSignal* _signal_symbol;
};

#define VALA_SIGNAL_TYPE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SIGNAL_TYPE, ValaSignalTypePrivate))
enum  {
	VALA_SIGNAL_TYPE_DUMMY_PROPERTY
};
static gboolean vala_signal_type_real_is_invokable (ValaDataType* base);
static ValaDataType* vala_signal_type_real_get_return_type (ValaDataType* base);
static GeeList* vala_signal_type_real_get_parameters (ValaDataType* base);
static ValaDataType* vala_signal_type_real_copy (ValaDataType* base);
static gpointer vala_signal_type_parent_class = NULL;
static void vala_signal_type_finalize (ValaCodeNode* obj);



ValaSignalType* vala_signal_type_construct (GType object_type, ValaSignal* signal_symbol) {
	ValaSignalType* self;
	g_return_val_if_fail (signal_symbol != NULL, NULL);
	self = (ValaSignalType*) g_type_create_instance (object_type);
	vala_signal_type_set_signal_symbol (self, signal_symbol);
	return self;
}


ValaSignalType* vala_signal_type_new (ValaSignal* signal_symbol) {
	return vala_signal_type_construct (VALA_TYPE_SIGNAL_TYPE, signal_symbol);
}


static gboolean vala_signal_type_real_is_invokable (ValaDataType* base) {
	ValaSignalType * self;
	self = (ValaSignalType*) base;
	return TRUE;
}


static ValaDataType* vala_signal_type_real_get_return_type (ValaDataType* base) {
	ValaSignalType * self;
	ValaDataType* _tmp0;
	self = (ValaSignalType*) base;
	_tmp0 = NULL;
	return (_tmp0 = vala_signal_get_return_type (self->priv->_signal_symbol), (_tmp0 == NULL) ? NULL : vala_code_node_ref (_tmp0));
}


static GeeList* vala_signal_type_real_get_parameters (ValaDataType* base) {
	ValaSignalType * self;
	self = (ValaSignalType*) base;
	return vala_signal_get_parameters (self->priv->_signal_symbol);
}


static ValaDataType* vala_signal_type_real_copy (ValaDataType* base) {
	ValaSignalType * self;
	self = (ValaSignalType*) base;
	return (ValaDataType*) vala_signal_type_new (self->priv->_signal_symbol);
}


ValaSignal* vala_signal_type_get_signal_symbol (ValaSignalType* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_signal_symbol;
}


void vala_signal_type_set_signal_symbol (ValaSignalType* self, ValaSignal* value) {
	ValaSignal* _tmp2;
	ValaSignal* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_signal_symbol = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1)), (self->priv->_signal_symbol == NULL) ? NULL : (self->priv->_signal_symbol = (vala_code_node_unref (self->priv->_signal_symbol), NULL)), _tmp2);
}


static void vala_signal_type_class_init (ValaSignalTypeClass * klass) {
	vala_signal_type_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_NODE_CLASS (klass)->finalize = vala_signal_type_finalize;
	g_type_class_add_private (klass, sizeof (ValaSignalTypePrivate));
	VALA_DATA_TYPE_CLASS (klass)->is_invokable = vala_signal_type_real_is_invokable;
	VALA_DATA_TYPE_CLASS (klass)->get_return_type = vala_signal_type_real_get_return_type;
	VALA_DATA_TYPE_CLASS (klass)->get_parameters = vala_signal_type_real_get_parameters;
	VALA_DATA_TYPE_CLASS (klass)->copy = vala_signal_type_real_copy;
}


static void vala_signal_type_instance_init (ValaSignalType * self) {
	self->priv = VALA_SIGNAL_TYPE_GET_PRIVATE (self);
}


static void vala_signal_type_finalize (ValaCodeNode* obj) {
	ValaSignalType * self;
	self = VALA_SIGNAL_TYPE (obj);
	(self->priv->_signal_symbol == NULL) ? NULL : (self->priv->_signal_symbol = (vala_code_node_unref (self->priv->_signal_symbol), NULL));
	VALA_CODE_NODE_CLASS (vala_signal_type_parent_class)->finalize (obj);
}


GType vala_signal_type_get_type (void) {
	static GType vala_signal_type_type_id = 0;
	if (vala_signal_type_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSignalTypeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_signal_type_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSignalType), 0, (GInstanceInitFunc) vala_signal_type_instance_init, NULL };
		vala_signal_type_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaSignalType", &g_define_type_info, 0);
	}
	return vala_signal_type_type_id;
}




