/* valavaluetype.vala
 *
 * Copyright (C) 2007-2009  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <vala/valavaluetype.h>
#include <vala/valastruct.h>
#include <vala/valacodenode.h>




struct _ValaValueTypePrivate {
	ValaTypeSymbol* _type_symbol;
};

#define VALA_VALUE_TYPE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_VALUE_TYPE, ValaValueTypePrivate))
enum  {
	VALA_VALUE_TYPE_DUMMY_PROPERTY
};
static char* vala_value_type_real_get_cname (ValaDataType* base);
static gboolean vala_value_type_real_is_disposable (ValaDataType* base);
static gboolean vala_value_type_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer);
static gpointer vala_value_type_parent_class = NULL;
static void vala_value_type_finalize (ValaCodeNode* obj);



ValaValueType* vala_value_type_construct (GType object_type, ValaTypeSymbol* type_symbol) {
	ValaValueType* self;
	g_return_val_if_fail (type_symbol != NULL, NULL);
	self = (ValaValueType*) g_type_create_instance (object_type);
	vala_value_type_set_type_symbol (self, type_symbol);
	vala_data_type_set_data_type ((ValaDataType*) self, type_symbol);
	return self;
}


ValaValueType* vala_value_type_new (ValaTypeSymbol* type_symbol) {
	return vala_value_type_construct (VALA_TYPE_VALUE_TYPE, type_symbol);
}


static char* vala_value_type_real_get_cname (ValaDataType* base) {
	ValaValueType * self;
	char* ptr;
	char* _tmp1;
	char* _tmp2;
	char* _tmp3;
	self = (ValaValueType*) base;
	ptr = g_strdup ("");
	if (vala_data_type_get_nullable ((ValaDataType*) self)) {
		char* _tmp0;
		_tmp0 = NULL;
		ptr = (_tmp0 = g_strdup ("*"), ptr = (g_free (ptr), NULL), _tmp0);
	}
	_tmp1 = NULL;
	_tmp2 = NULL;
	_tmp3 = NULL;
	return (_tmp3 = (_tmp2 = g_strconcat (_tmp1 = vala_typesymbol_get_cname (self->priv->_type_symbol, FALSE), ptr, NULL), _tmp1 = (g_free (_tmp1), NULL), _tmp2), ptr = (g_free (ptr), NULL), _tmp3);
}


static gboolean vala_value_type_real_is_disposable (ValaDataType* base) {
	ValaValueType * self;
	ValaStruct* _tmp3;
	ValaTypeSymbol* _tmp2;
	ValaStruct* st;
	gboolean _tmp5;
	self = (ValaValueType*) base;
	if (!vala_data_type_get_value_owned ((ValaDataType*) self)) {
		return FALSE;
	}
	/* nullable structs are heap allocated*/
	if (vala_data_type_get_nullable ((ValaDataType*) self)) {
		return TRUE;
	}
	_tmp3 = NULL;
	_tmp2 = NULL;
	st = (_tmp3 = (_tmp2 = self->priv->_type_symbol, VALA_IS_STRUCT (_tmp2) ? ((ValaStruct*) _tmp2) : NULL), (_tmp3 == NULL) ? NULL : vala_code_node_ref (_tmp3));
	if (st != NULL) {
		gboolean _tmp4;
		return (_tmp4 = vala_struct_is_disposable (st), (st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL)), _tmp4);
	}
	return (_tmp5 = FALSE, (st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL)), _tmp5);
}


static gboolean vala_value_type_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer) {
	ValaValueType * self;
	self = (ValaValueType*) base;
	g_return_val_if_fail (analyzer != NULL, FALSE);
	return vala_code_node_check ((ValaCodeNode*) self->priv->_type_symbol, analyzer);
}


ValaTypeSymbol* vala_value_type_get_type_symbol (ValaValueType* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_type_symbol;
}


void vala_value_type_set_type_symbol (ValaValueType* self, ValaTypeSymbol* value) {
	g_return_if_fail (self != NULL);
	self->priv->_type_symbol = value;
}


static void vala_value_type_class_init (ValaValueTypeClass * klass) {
	vala_value_type_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_NODE_CLASS (klass)->finalize = vala_value_type_finalize;
	g_type_class_add_private (klass, sizeof (ValaValueTypePrivate));
	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_value_type_real_get_cname;
	VALA_DATA_TYPE_CLASS (klass)->is_disposable = vala_value_type_real_is_disposable;
	VALA_CODE_NODE_CLASS (klass)->check = vala_value_type_real_check;
}


static void vala_value_type_instance_init (ValaValueType * self) {
	self->priv = VALA_VALUE_TYPE_GET_PRIVATE (self);
}


static void vala_value_type_finalize (ValaCodeNode* obj) {
	ValaValueType * self;
	self = VALA_VALUE_TYPE (obj);
	VALA_CODE_NODE_CLASS (vala_value_type_parent_class)->finalize (obj);
}


GType vala_value_type_get_type (void) {
	static GType vala_value_type_type_id = 0;
	if (vala_value_type_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaValueTypeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_value_type_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaValueType), 0, (GInstanceInitFunc) vala_value_type_instance_init, NULL };
		vala_value_type_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaValueType", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return vala_value_type_type_id;
}




